# -*- coding: utf-8 -*-

#chnage default directory
import os
os.chdir("C:/data")

#load the dataset
import pandas
dfAdj = pandas.read_table("karate.txt",header=0,index_col=0)

#show various information about the dataset
n = dfAdj.shape[0]
print(n)
print(dfAdj.info())
print(dfAdj.index)

#transform the DataFrame into a Numpy Matrix
MAdj = dfAdj.as_matrix()
print(MAdj)

#import the igraph package
import igraph
print(igraph.__version__)

#create the graphe
g = igraph.Graph.Adjacency(MAdj.tolist(),mode=igraph.ADJ_UNDIRECTED)
print(g)

#assign names to vertices
g.vs["name"] = dfAdj.index.tolist()
print(g.vs[10]["name"])

#number of neighbors for p11
print(g.neighborhood_size("p11"))

#neighbors of p11
print(g.neighborhood("p11"))

#displaying their names
print(g.vs[g.neighborhood("p11")]['name'])

#assign labels to vertices
g.vs['label'] = dfAdj.index.tolist()
print(g.vs[0]['label'])

#plotting the graph
obj = igraph.plot(g,vertex_label_size=15,vertex_size=35,vertex_color='#ffe4c4')
obj.show()

#degree of centrality
print(g.vs.degree())

#displaying the 5 first vertices, sorting decreasing according the degree of centrality
valeurs = {'node_name':g.vs['name'],'degree':g.vs.degree()}
print(pandas.DataFrame.from_dict(valeurs).sort_values(by='degree',ascending=False).iloc[:5,:])

#displaying the 5 first vertices, sorting decreasing according the closeness centrality
valeurs = {'node_name':g.vs['name'],'closeness':g.vs.closeness()}
print(pandas.DataFrame.from_dict(valeurs).sort_values(by='closeness',ascending=False).iloc[:5,:])

#displaying the 5 first vertices, sorting decreasing according the betweenness centrality
valeurs = {'node_name':g.vs['name'],'betweenness':g.vs.betweenness()}
print(pandas.DataFrame.from_dict(valeurs).sort_values(by='betweenness',ascending=False).iloc[:5,:])

#graph where p31 and p34 are highlighted
g.vs['color'] = '#ffe4c4'
g.vs[0]['color'] = '#1e90ff'
g.vs[33]['color'] = 'green'
obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()

#the shortests paths between p1 and p34
print(g.get_all_shortest_paths('p1','p34'))

#highlight the intermediate vertices between p1 and p34
for v in g.get_all_shortest_paths('p1','p34'):
    g.vs[v[1]]['color'] = '#ffa07a'

obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()

#edge betweenness values
eb = g.edge_betweenness()
print(eb)

#transform into a numpy vector
import numpy
ebn = numpy.array(eb)
print(ebn)

#index of the maximum value
iebn = numpy.argmax(ebn)
print(iebn)

#highlight the concerned edge
g.vs['color'] = '#ffe4c4'
g.vs[0]['color'] = '#1e90ff'
g.vs[33]['color'] = 'green'
g.es['width'] = 1
g.es[iebn]['width'] = 4
obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()

#community detection with the edge betweenness approach
res = g.community_edge_betweenness(directed=False)

#number of groups detected by the approach
print(res.optimal_count)

#displaying the dendrogram
igraph.plot(res)

#cutting into two groups
sol = res.as_clustering(n=2)
print(sol)

#highlighting the groups into the graph
g.vs[sol[0]]['color'] = '#87ceee'
g.vs[0]['color'] = '#1e90ff'
g.vs[sol[1]]['color'] = '#90dd90'
g.vs[33]['color'] = 'green'
g.es['width'] = 1
obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()
