# -*- coding: utf-8 -*-

#changement de répertoire
import os
os.chdir("C:/data")

#chargement des données
import pandas
dfAdj = pandas.read_table("karate.txt",header=0,index_col=0)

#affichage des informations
n = dfAdj.shape[0]
print(n)
print(dfAdj.info())
print(dfAdj.index)

#transformer en matrice
MAdj = dfAdj.as_matrix()
print(MAdj)

#igraph
import igraph
print(igraph.__version__)

#création du graphe
g = igraph.Graph.Adjacency(MAdj.tolist(),mode=igraph.ADJ_UNDIRECTED)
print(g)

#attribuer des noms aux sommets
g.vs["name"] = dfAdj.index.tolist()
print(g.vs[10]["name"])

#nombre de voisins de p11
print(g.neighborhood_size("p11"))

#voisinage de p11
print(g.neighborhood("p11"))

#si on souhaite avoir leurs noms
print(g.vs[g.neighborhood("p11")]['name'])

#attribuer des étiquettes aux sommets
g.vs['label'] = dfAdj.index.tolist()
print(g.vs[0]['label'])

#affichage du graphe
obj = igraph.plot(g,vertex_label_size=15,vertex_size=35,vertex_color='#ffe4c4')
obj.show()

#degré de centralité
print(g.vs.degree())

#affichage des 5 premiers sommets, tri de manière décroissante
valeurs = {'node_name':g.vs['name'],'degree':g.vs.degree()}
print(pandas.DataFrame.from_dict(valeurs).sort_values(by='degree',ascending=False).iloc[:5,:])

#closeness centralité
valeurs = {'node_name':g.vs['name'],'closeness':g.vs.closeness()}
print(pandas.DataFrame.from_dict(valeurs).sort_values(by='closeness',ascending=False).iloc[:5,:])

#betweenness centralité
valeurs = {'node_name':g.vs['name'],'betweenness':g.vs.betweenness()}
print(pandas.DataFrame.from_dict(valeurs).sort_values(by='betweenness',ascending=False).iloc[:5,:])

#graphe mettant en évidence p31 et p34
g.vs['color'] = '#ffe4c4'
g.vs[0]['color'] = '#1e90ff'
g.vs[33]['color'] = 'green'
obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()

#plus court chemin entre p1 et p34
print(g.get_all_shortest_paths('p1','p34'))

#mettre en évidence les individus relais
for v in g.get_all_shortest_paths('p1','p34'):
    g.vs[v[1]]['color'] = '#ffa07a'

obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()

#récupération des edge betweenness
eb = g.edge_betweenness()
print(eb)

#transofrmer en vecteur numpy
import numpy
ebn = numpy.array(eb)
print(ebn)

#récupérer l'indice la valeur maximale
iebn = numpy.argmax(ebn)
print(iebn)

#mettre en évidence la connexion
g.vs['color'] = '#ffe4c4'
g.vs[0]['color'] = '#1e90ff'
g.vs[33]['color'] = 'green'
g.es['width'] = 1
g.es[iebn]['width'] = 4
obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()

#construction de communautés avec edge betweenness
res = g.community_edge_betweenness(directed=False)

#nombre de groupes proposés
print(res.optimal_count)

#affichage du dendrogramme
igraph.plot(res)

#couper en 2 groupes
sol = res.as_clustering(n=2)
print(sol)

#représenter les groupes dans le graphe
g.vs[sol[0]]['color'] = '#87ceee'
g.vs[0]['color'] = '#1e90ff'
g.vs[sol[1]]['color'] = '#90dd90'
g.vs[33]['color'] = 'green'
g.es['width'] = 1
obj = igraph.plot(g,vertex_label_color='black',vertex_label_size=15,vertex_size=35)
obj.show()
